<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 11/26/2008 *
 * ===================                          Last change: 11/26/2008 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : ext-booking.php                                  *
 * -------------------------------------------------------------------- *
 * Short description : User bookings                                    *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Kontoauszug der Mitglieder                       *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2015 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} // END - if

// Version number
setThisExtensionVersion('0.0.1');

// Version history array (add more with , '0.0.1' and so on)
setExtensionVersionHistory(array('0.0.0', '0.0.1'));

// Check extension load-mode
switch (getExtensionMode()) {
	case 'setup': // Do stuff when installation is running
		// Configuration entries
		addConfigAddSql('booking_page_count', 'TINYINT(3) UNSIGNED NOT NULL DEFAULT 10');
		addConfigAddSql('booking_purge', 'BIGINT(20) UNSIGNED NOT NULL DEFAULT ' . (getOneDay() * 3));

		// Drop/create table for user bookings
		addDropTableSql('user_booking');
		addCreateTableSql('user_booking', "
`booking_id` BIGINT(20) UNSIGNED NOT NULL AUTO_INCREMENT,
`userid` BIGINT(20) UNSIGNED NOT NULL DEFAULT 0,
`subject` VARCHAR(255) NOT NULL DEFAULT 'missing',
`points_mode` ENUM('ADD','SUB') NOT NULL DEFAULT 'ADD',
`points_account_type` VARCHAR(255) NOT NULL DEFAULT 'INVALID',
`points` FLOAT(20,5) UNSIGNED NOT NULL DEFAULT 0.0000,
`comments` TINYTEXT NULL DEFAULT NULL,
`recorded` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
PRIMARY KEY (`booking_id`),
INDEX (`userid`)",
			'Member points booking table');

		// Admin menu
		addAdminMenuSql('setup','config_booking','Kontoauszug','Einstellungen am Kontoauszug f&uuml;r Mitglieder vornehmen.', 14);
		addAdminMenuSql('user','list_booking','Kontoausz&uuml;ge','Kontoausz&uuml;ge aller Ihrer Mitglieder oder eines einzelnen Mitgliedes anzeigen.',10);

		// Member menu
		addMemberMenuSql('main', 'booking', '{OPEN_CONFIG}POINTS{CLOSE_CONFIG}-Kontoauszug', 5);

		// Add the filters
		registerFilter(__FILE__, __LINE__, 'post_add_points', 'ADD_BOOKING_RECORD', FALSE, TRUE, isExtensionDryRun());
		registerFilter(__FILE__, __LINE__, 'post_sub_points', 'ADD_BOOKING_RECORD', FALSE, TRUE, isExtensionDryRun());
		registerFilter(__FILE__, __LINE__, 'member_admin_actions', 'ADD_BOOKING_MEMBER_ACTION', FALSE, TRUE, isExtensionDryRun());
		break;

	case 'remove': // Do stuff when removing extension
		addDropTableSql('user_booking');
		addExtensionSql("DELETE LOW_PRIORITY FROM `{?_MYSQL_PREFIX?}_admin_menu` WHERE `what` IN ('config_booking','list_booking')");
		addExtensionSql("DELETE LOW_PRIORITY FROM `{?_MYSQL_PREFIX?}_member_menu` WHERE `what`='booking'");

		// Remove the filters
		unregisterFilter(__FILE__, __LINE__, 'post_add_points', 'ADD_BOOKING_RECORD', TRUE, isExtensionDryRun());
		unregisterFilter(__FILE__, __LINE__, 'post_sub_points', 'ADD_BOOKING_RECORD', TRUE, isExtensionDryRun());
		unregisterFilter(__FILE__, __LINE__, 'member_admin_actions', 'ADD_BOOKING_MEMBER_ACTION', TRUE, isExtensionDryRun());
		break;

	case 'activate': // Do stuff when admin activates this extension
		// SQL commands to run
		addExtensionSql("UPDATE `{?_MYSQL_PREFIX?}_member_menu` SET `visible`='Y',`locked`='N' WHERE `what`='booking' LIMIT 1");
		break;

	case 'deactivate': // Do stuff when admin deactivates this extension
		// SQL commands to run
		addExtensionSql("UPDATE `{?_MYSQL_PREFIX?}_member_menu` SET `visible`='N',`locked`='Y' WHERE `what`='booking' LIMIT 1");
		break;

	case 'update': // Update an extension
		switch (getCurrentExtensionVersion()) {
			case '0.0.1': // SQL queries for v0.0.1
				addExtensionChangeTableColumnSql('user_booking', 'id', 'booking_id', 'BIGINT(20) UNSIGNED NOT NULL AUTO_INCREMENT');

				// Update notes (these will be set as task text!)
				setExtensionUpdateNotes("Spalte <strong>id</strong umbenannt nach <strong>booking_id</strong>, da <em>id</em> zu allgemein ist (sollte am besten &uuml;berall folgen).");
				break;
		} // END - switch

	case 'modify': // When the extension got modified
		break;

	case 'test': // For testing purposes
		break;

	case 'init': // Do stuff when extension is initialized
		break;

	default: // Unknown extension mode
		reportBug(__FILE__, __LINE__, sprintf('Unknown extension mode %s in extension %s detected.', getExtensionMode(), getCurrentExtensionName()));
		break;
} // END - switch

// [EOF]
?>
