<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 12/16/2008 *
 * ===================                          Last change: 12/16/2008 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : filters.php                                      *
 * -------------------------------------------------------------------- *
 * Short description : Generic filters                                  *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Allgemeine Filter                                *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2015 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} // END - if

// Filter for flushing all new filters to the database
function FILTER_FLUSH_FILTERS () {
	// Clear all previous SQL queries
	initSqls();

	// Is a database link here and not in installation mode?
	if ((!isSqlLinkUp()) && (!isInstaller())) {
		// Abort here
		reportBug(__FUNCTION__, __LINE__, 'No database link is up. Cannot flush filters.');
	} elseif ((isInstaller()) && (!isSqlLinkUp())) {
		// If the link is not up in installation phase, skip flushing filters
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'Cannot flush filters: No link is up.');
		return;
	}

	// Is the extension ext-sql_patches updated?
	if (((!isExtensionInstalled('sql_patches')) || (isExtensionInstalledAndOlder('sql_patches', '0.5.9'))) && (!isInstaller())) {
		// Log only in debug mode
		if (isDebugModeEnabled()) {
			logDebugMessage(__FUNCTION__, __LINE__, 'Cannot flush filters, ext-sql_patches might be missing. isExtensionInstalled()=' . intval(isExtensionInstalled('sql_patches')) . ',isExtensionInstalledAndOlder()=' . intval(isExtensionInstalledAndOlder('sql_patches', '0.5.9')));
		} // END - if

		// Abort silently here
		return FALSE;
	} // END - if

	// Nothing is added/remove by default
	$inserted = '0';
	$removed = '0';

	// Prepare SQL queries
	$insertSQL = 'INSERT INTO `{?_MYSQL_PREFIX?}_filters` (`filter_name`, `filter_function`, `filter_active`) VALUES';
	$removeSQL = 'DELETE LOW_PRIORITY FROM `{?_MYSQL_PREFIX?}_filters` WHERE';

	// Write all filters to database
	foreach ($GLOBALS['cache_array']['filter']['chains'] as $filterName => $filterArray) {
		// Walk through all filters
		foreach ($filterArray as $filterFunction => $active) {
			// Is this filter loaded?
			//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'FOUND:'.$filterName.'/'.$filterFunction.'='.$active);
			if (((!isset($GLOBALS['cache_array']['filter']['loaded'][$filterName][$filterFunction])) && ($active != 'R')) || ($active == 'A')) {
				// Is this filter already in database?
				if (countSumTotalData($filterFunction, 'filters', 'filter_name', 'filter_function', TRUE, sprintf(" AND `filter_name`='%s'", sqlEscapeString($filterName))) == 0) {
					// Add this filter (all filters are active by default)
					//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'ADD:'.$filterName.'/'.$filterFunction);
					$insertSQL .= sprintf("('%s','%s','Y'),", $filterName, $filterFunction);
					$inserted++;
				} elseif (isDebugModeEnabled()) {
					// Log already registered filters
					logDebugMessage(__FUNCTION__, __LINE__, 'Already registered: filterName=' . $filterName . ',filterFunction=' . $filterFunction . ',active=' . $active);
				}
			} elseif ($active == 'R') {
				// Remove this filter
				//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'REMOVE:'.$filterName.'/'.$filterFunction);
				$removeSQL .= sprintf(" (`filter_name`='%s' AND `filter_function`='%s') OR", $filterName, $filterFunction);
				$removed++;
			}
		} // END - foreach
	} // END - foreach

	// Debug message
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'inserted=' . $inserted . ',removed=' . $removed . ' - AFTER COLLECTION');

	// Something has been added?
	if ($inserted > 0) {
		// Finish SQL command and add it
		addSql(substr($insertSQL, 0, -1));
	} // END - if

	// Something has been removed?
	if ($removed > 0) {
		// Finish SQL command and add it
		addSql(substr($removeSQL, 0, -2) . 'LIMIT ' . $removed);
	} // END - if

	// Shall we update usage counters (ONLY FOR DEBUGGING!)
	if (isFilterUsageUpdateEnabled()) {
		// Update all counters
		foreach ($GLOBALS['cache_array']['filter']['counter'] as $filterName => $filterArray) {
			// Walk through all filters
			foreach ($filterArray as $filterFunction => $count) {
				// Construct and add the query
				addSql(sprintf("UPDATE `{?_MYSQL_PREFIX?}_filters` SET `filter_counter`=%s WHERE `filter_name`='%s' AND `filter_function`='%s' LIMIT 1",
					bigintval($count),
					$filterName,
					$filterFunction
				));
			} // END - foreach
		} // END - foreach
	} // END - if

	// Run the run_sqls filter in non-dry mode
	runFilterChain('run_sqls');

	// Should we rebuild cache?
	if (($inserted > 0) || ($removed > 0)) {
		// Destroy cache
		rebuildCache('filter', 'filter');
	} // END - if
}

// Filter for calling the handler for login failures
function FILTER_CALL_HANDLER_LOGIN_FAILTURES ($filterData) {
	// Init content
	$content = $filterData;

	// Handle failed logins here if not in guest
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'type=' . $filterData['type'] . ',action=' . getAction() . ',what=' . getWhat() . ',level=' . $filterData['access_level']);
	if ((($filterData['type'] == 'what') || ($filterData['type'] == 'action') && ((!isWhatSet()) || (getWhat() == 'welcome') || (getWhat() == getIndexHome()))) && ($filterData['access_level'] != 'guest') && ((isExtensionInstalledAndNewer('sql_patches', '0.4.7')) || (isExtensionInstalledAndNewer('admins', '0.7.6')))) {
		// Handle failure
		$content['content'] .= handleLoginFailures($filterData['access_level']);
	} // END - if

	// Return the content
	return $content;
}

// Filter for auto-activation of a extension
function FILTER_AUTO_ACTIVATE_EXTENSION ($filterData) {
	// Debug message
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'ext_name=' . $filterData['ext_name'] . ',isThisExtensionAlwaysActive()=' . intval(isThisExtensionAlwaysActive()));

	// Is this extension always activated?
	if (isThisExtensionAlwaysActive()) {
		// Then activate the extension
		doActivateExtension($filterData['ext_name']);
	} // END - if

	// Return the data
	return $filterData;
}

// Filter for solving task given task
function FILTER_SOLVE_TASK ($filterData) {
	// Don't solve anything if no admin!
	if (!isAdmin()) {
		return $filterData;
	} // END - if

	// Is this a direct task id or array element task_id is found?
	if (is_int($filterData)) {
		// Then solve it...
		adminSolveTask($filterData);
	} elseif ((is_array($filterData)) && (isset($filterData['task_id']))) {
		// Solve it...
		adminSolveTask($filterData['task_id']);
	} else {
		// Not detectable!
		reportBug(__FUNCTION__, __LINE__, sprintf('Cannot resolve task. data[%s]=<pre>%s</pre>', gettype($filterData), print_r($filterData, TRUE)));
	}

	// Return the data
	return $filterData;
}

// Filter to load include files
function FILTER_LOAD_INCLUDES ($filterData) {
	// Is it null?
	if (is_null($filterData)) {
		// This should not happen!
		reportBug(__FUNCTION__, __LINE__, 'pool is null.');
	} // END - if

	// Is the pool an array and 'pool' set?
	if ((is_array($filterData)) && (isset($filterData['pool']))) {
		// Then use it as pool
		$realPool = $filterData['pool'];
	} else {
		// Default is $data as inclusion list
		$realPool = $filterData;
	}

	// Get inc pool
	$data = getIncludePool($realPool);

	// Is it an array?
	if ((!isset($data)) || (!is_array($data))) {
		// Then abort here
		reportBug(__FUNCTION__, __LINE__, sprintf('INC_POOL is no array! Type: %s', gettype($data)));
	} elseif (isset($data['inc_pool'])) {
		// Use this as new inclusion pool!
		setIncludePool($realPool, $data['inc_pool']);
	}

	// Check for added include files
	if (countIncludePool($realPool) > 0) {
		// Loads every include file
		loadIncludePool($realPool);

		// Reset array
		initIncludePool($realPool);
	} // END - if

	// Continue with processing
	return $filterData;
}

// Filter for running SQL commands
function FILTER_RUN_SQLS ($filterData) {
	// Init counter
	$count = 0;

	// Debug message
	//* DEBUG: */ die('getSqls()=<pre>'.print_r(getSqls(), TRUE).'</pre>ext_update_depends=<pre>'.print_r($GLOBALS['ext_update_depends'], TRUE).'</pre>');
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, '- Entered!');

	// Is the array there?
	if ((ifSqlsRegistered()) && ((!isset($filterData['dry_run'])) || ($filterData['dry_run'] == FALSE))) {
		// Run SQL commands
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, '- Found ' . countSqls() . ' queries to run.');
		foreach (getSqls() as $mode => $sqls) {
			// Debug message
			//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'mode=' . $mode . ',count()=' . count($sqls));

			// New cache format...
			foreach ($sqls as $sql) {
				// Trim spaces away
				$sql = trim($sql);

				// Is 'enable_codes' not set? Then set it to true
				if (!isset($filterData['enable_codes'])) {
					$filterData['enable_codes'] = TRUE;
				} // END - if

				// Is there still a query left?
				if (!empty($sql)) {
					// Is there an "ALTER TABLE" command? Exclude installation phase here as it would cause this query to fail
					if ((substr(strtolower($sql), 0, 11) == 'alter table') && (!isset($filterData['ext_installing']))) {
						// Analyse the alteration command
						//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'Alterting table: ' . $sql . ',enable_codes=' . intval($filterData['enable_codes']));
						sqlQueryAlterTable($sql, __FUNCTION__, __LINE__, $filterData['enable_codes']);
					} else {
						// Run regular SQL command
						//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'Running regular query: ' . $sql . ',enable_codes=' . intval($filterData['enable_codes']));
						sqlQuery($sql, __FUNCTION__, __LINE__, $filterData['enable_codes']);
					}

					// Has a line changed?
					if (!ifSqlHasZeroAffectedRows()) {
						// Add it to total account
						$count += sqlAffectedRows();
					} // END - if
				} // END - if
			} // END - foreach
		} // END - foreach
	} // END - if

	// Debug message
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, '- Left!');

	// Return counter
	return $count;
}

// Filter for updating/validating login data
function FILTER_UPDATE_LOGIN_DATA () {
	// Add missing array
	if ((!isset($GLOBALS['last_online'])) || (!is_array($GLOBALS['last_online']))) {
		$GLOBALS['last_online'] = array();
	} // END - if

	// Recheck if logged in
	if ((!isMember()) || ((isAdmin()) && (!isDebugModeEnabled()))) {
		return FALSE;
	} // END - if

	// Secure user id
	setMemberId(getSession('userid'));

	// Found a userid?
	if (fetchUserData(getMemberId())) {
		// Load last module and online time
		$content = getUserDataArray();

		// Maybe first login time?
		if (empty($content[getUserLastWhatName()])) {
			$content[getUserLastWhatName()] = 'login';
		} // END - if

		// This will be displayed on welcome page! :-)
		if (empty($GLOBALS['last_online']['module'])) {
			$GLOBALS['last_online']['module'] = $content[getUserLastWhatName()];
			$GLOBALS['last_online']['online'] = $content['last_online'];
		} // END - if

		// 'what' not set?
		if (!isWhatSet()) {
			// Fix it to default
			setWhat('welcome');
			if (getIndexHome() != '') {
				setWhatFromConfig('index_home');
			} // END - if
		} // END - if

		// Update last module / online time
		updateLastActivity(getMemberId());
	}  else {
		// Destroy session, we cannot update!
		destroyMemberSession();
	}
}

// Filter for initializing randomizer
function FILTER_INIT_RANDOMIZER () {
	// Take a prime number which is long (if you know a longer one please try it out!)
	setConfigEntry('_PRIME', 591623);

	// Initialize array with all allowed chars
	$GLOBALS['_abc'] = explode(',', 'a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v,w,x,y,z,A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S,T,U,V,W,X,Y,Z,1,2,3,4,6,7,8,9,-,+,_,/,.');

	// Only execute this filter if installed
	if ((!isInstalled()) || (!isExtensionInstalledAndNewer('other', '0.2.5'))) {
		// Set missing entry
		setConfigEntry('_ADD', 0);
		return;
	} // END - if

	// Calculate "entropy" with the prime number (for code generation)
	setConfigEntry('_ADD', (getPrime() * getPrime() / (pi() * getCodeLength() + 1)));

	// Simply init the randomizer with seed and _ADD value
	mt_srand(generateSeed() + getConfig('_ADD'));
}

// Filter for removing updates
function FILTER_REMOVE_UPDATES ($filterData) {
	// Init removal list
	initExtensionRemovalList();

	// Add the current extension to it
	addCurrentExtensionToRemovalList();

	// Simply remove it
	unsetExtensionSqls();

	// Are there update dependencies to remove?
	if (countExtensionUpdateDependencies() > 0) {
		// Then find all updates we shall no longer execute
		foreach (getExtensionUpdateDependencies() as $id => $ext_name) {
			// Shall we remove this update?
			if (in_array($ext_name, getExtensionRemovalList())) {
				// Then remove this extension!
				removeExtensionDependency($ext_name);
			} // END - if
		} // END - foreach
	} // END - if

	// Return data
	return $filterData;
}

// Determines username for current user state
function FILTER_DETERMINE_USERNAME () {
	// Check if logged in
	if (isMember()) {
		// Is still logged in so we welcome him with his name
		if (fetchUserData(getMemberId())) {
			// Load surname and family's name and build the username
			$content = getUserDataArray();

			// Prepare username
			setUsername($content['surname'] . ' ' . $content['family']);

			// Additionally admin?
			if (isAdmin()) {
				// Add it
				setUsername(getUsername() . ' ({--USERNAME_ADMIN_SHORT--})');
			} // END - if
		} else {
			// Hmmm, logged in and no valid userid?
			setUsername('<span class="bad">{--USERNAME_UNKNOWN--}</span>');

			// Destroy session
			destroyMemberSession();
		}
	} elseif (isAdmin()) {
		// Admin is there
		setUsername('{--USERNAME_ADMIN--}');
	} else {
		// He's a guest, hello there... ;-)
		setUsername('{--USERNAME_GUEST--}');
	}
}

// Filter for compiling config entries
function FILTER_COMPILE_CONFIG ($code, $compiled = FALSE) {
	// Save the uncompiled code
	$uncompiled = $code;

	// Is there cache?
	if (!isset($GLOBALS['compiled_config'][$code])) {
		// Compile {?some_var?} to getConfig('some_var')
		preg_match_all('/\{\?(([a-zA-Z0-9-_]+)*)\?\}/', $code, $matches);

		// Some entries found?
		if ((isFilledArray($matches)) && (isFilledArray($matches[0]))) {
			// Replace all matches
			foreach ($matches[0] as $key => $match) {
				// Is there cache?
				if (!isset($GLOBALS['compile_config'][$matches[1][$key]])) {
					// Is the config valid?
					if (isConfigEntrySet($matches[1][$key])) {
						// Set it for caching
						$GLOBALS['compile_config'][$matches[1][$key]] = '{%config=' . $matches[1][$key] . '%}';
					} elseif (isConfigEntrySet('default_' . strtoupper($matches[1][$key]))) {
						// Use default value
						$GLOBALS['compile_config'][$matches[1][$key]] = '{%config=' . 'DEFAULT_' . strtoupper($matches[1][$key]) . '%}';
					} elseif (isMessageIdValid('DEFAULT_' . strtoupper($matches[1][$key]))) {
						// No config, try the language system
						$GLOBALS['compile_config'][$matches[1][$key]] = '{%message,DEFAULT_' . strtoupper($matches[1][$key]) . '%}';
					} else {
						// Unhandled!
						$GLOBALS['compile_config'][$matches[1][$key]] = '!' . $matches[1][$key] . '!';
					}
				} // END - if

				// Use this for replacing
				$code = str_replace($match, $GLOBALS['compile_config'][$matches[1][$key]], $code);
				//* DEBUG: */ if (($match == '{?URL?}') && (strlen($code) > 10000)) die(__FUNCTION__.'['.__LINE__.']:<pre>'.secureString($code).'</pre>');
			} // END - foreach
		} // END - if

		// Add it to cache
		$GLOBALS['compiled_config'][$uncompiled] = $code;
	} // END - if

	// Should we compile it?
	if ($compiled === TRUE) {
		// Run the code
		$eval = "\$GLOBALS['compiled_config'][\$uncompiled] = \"" . $GLOBALS['compiled_config'][$uncompiled] . '";';
		//* DEBUG: */ debugOutput(__FUNCTION__.'['.__LINE__.']:<pre>' . encodeEntities($eval) . '</pre>');
		eval($eval);
	} // END - if

	// Return compiled code
	return $GLOBALS['compiled_config'][$uncompiled];
}

// Filter for compiling expression code
function FILTER_COMPILE_EXPRESSION_CODE ($filterData) {
	// Compile {%cmd,callback,extraFunction=some_value%} to get expression code snippets
	// See switch() command below for supported commands
	preg_match_all('/\{%(([a-zA-Z0-9-_,]+)(=([^\}]+)){0,1})*%\}/', $filterData, $matches);
	//* DEBUG: */ debugOutput(__FUNCTION__.'['.__LINE__.']:<pre>'.print_r($matches, TRUE).'</pre>');

	// Default is from outputHtml()
	$outputMode = getScriptOutputMode();

	// Some entries found?
	if ((isFilledArray($matches)) && (isFilledArray($matches[3]))) {
		// Replace all matches
		foreach ($matches[2] as $key => $eti) {
			// Init replacer/call-back variable
			$replacer       = '';
			$callback       = '';
			$extraFunction  = '';
			$extraFunction2 = '';
			$value          = NULL;

			// Extract command and call-back
			$etiArray = explode(',', $eti);
			$eti = $etiArray[0];

			// Detect call-back function
			if (isset($etiArray[1])) {
				// Call-back function detected
				$callback = $etiArray[1];
			} // END - if

			// Detect extra function
			if (isset($etiArray[2])) {
				// Also detected
				$extraFunction = $etiArray[2];
			} // END - if

			// Detect extra function 2
			if (isset($etiArray[3])) {
				// Also detected
				$extraFunction2 = $etiArray[3];
			} // END - if

			// And value
			if (isset($matches[4][$key])) {
				// Use this as value
				$value = $matches[4][$key];
			} // END - if

			// Construct call-back function name for the command
			$etiFunction = 'doExpression' . capitalizeUnderscoreString($eti);

			// Is this function there?
			if (function_exists($etiFunction)) {
				// Prepare $matches, $key, $outputMode, etc.
				$data = array(
					'matches'     => $matches,
					'key'         => $key,
					'output_mode' => getScriptOutputMode(),
					'code'        => $filterData,
					'callback'    => $callback,
					'extra_func'  => $extraFunction,
					'extra_func2' => $extraFunction2,
					'value'       => $value
				);

				// Call it
				//* DEBUG: */ debugOutput(__FUNCTION__ . '[' . __LINE__ . ']: etiFunction=' . $etiFunction);
				$filterData = call_user_func($etiFunction, $data);
			} else {
				// Unsupported command detected
				logDebugMessage(__FUNCTION__, __LINE__, 'eti=' . $eti . ',etiFunction=' . $etiFunction . ',callback=' . $callback . ',extraFunction=' . $extraFunction . ',extraFunction2=' . $extraFunction2 . ' - UNSUPPORTED');
			}
		} // END - foreach
	} // END - if

	// Is there non-HTML/AJAX mode?
	if ((!isHtmlOutputMode()) && (!isAjaxOutputMode())) {
		$filterData = decodeEntities($filterData);
	} // END - if

	// Return compiled code
	//* DEBUG: */ debugOutput(__FUNCTION__.'['.__LINE__.']:<pre>'.($filterData).'</pre>');
	return $filterData;
}

// Runs some generic filter update steps
function FILTER_UPDATE_EXTENSION_DATA ($ext_name) {
	// Create task (we ignore the task id here)
	createExtensionUpdateTask(getCurrentAdminId(), $ext_name, $GLOBALS['update_ver'][$ext_name], sqlEscapeString(getExtensionNotes(getExtensionNotes())));

	// Update extension's version
	sqlQueryEscaped("UPDATE `{?_MYSQL_PREFIX?}_extensions` SET `ext_version`='%s' WHERE `ext_name`='%s' LIMIT 1",
		array($GLOBALS['update_ver'][$ext_name], $ext_name), __FUNCTION__, __LINE__);

	// Remove arrays
	unsetSqls();
	unset($GLOBALS['update_ver'][$ext_name]);
}

// Load more hourly reset scripts
function FILTER_RUN_HOURLY_INCLUDES () {
	// Is the reset set or old sql_patches?
	if (((!isHourlyResetEnabled()) || (!isExtensionInstalledAndNewer('sql_patches', '0.9.8'))) && (isHtmlOutputMode())) {
		// Then abort here
		reportBug(__FUNCTION__, __LINE__, 'Cannot run hourly reset! enabled='.intval(isHourlyResetEnabled()).',isExtensionInstalledAndNewer='.intval(isExtensionInstalledAndNewer('sql_patches', '0.9.8')).' Please report this bug. Thanks');
	} // END - if

	// Is ext-sql_patches at least 0.9.8?
	if (isExtensionInstalledAndNewer('sql_patches', '0.9.8')) {
		// Has it changed?
		if ((getLastHourly() != getHour()) || (isDebugHourlyEnabled())) {
			// Get more hourly reset scripts
			setIncludePool('hourly', getArrayFromDirectory('inc/hourly/', 'hourly_'));

		// Run the filter
		runFilterChain('load_includes', 'hourly');
		} // END - if
	} // END - if
}

// Load more daily scripts
function FILTER_RUN_DAILY_INCLUDES () {
	// Is the reset set or old sql_patches?
	if (((!isDailyResetEnabled()) || (!isExtensionInstalledAndNewer('sql_patches', '0.9.8'))) && (isHtmlOutputMode())) {
		// Then abort here
		reportBug(__FUNCTION__, __LINE__, 'Cannot run daily reset! enabled='.intval(isDailyResetEnabled()).',isExtensionInstalledAndNewer='.intval(isExtensionInstalledAndNewer('sql_patches', '0.9.8')).' Please report this bug. Thanks');
	} // END - if

	// Is ext-sql_patches at least 0.9.8?
	if (isExtensionInstalledAndNewer('sql_patches', '0.9.8')) {
		// Has it changed?
		if ((getLastDaily() != getDay()) || (isDebugDailyEnabled())) {
			// Get more daily reset scripts
			setIncludePool('daily', getArrayFromDirectory('inc/daily/', 'daily_'));

			// Run the filter
			runFilterChain('load_includes', 'daily');
		} // END - if
	} // END - if
}

// Load more weeklt scripts
function FILTER_RUN_WEEKLY_INCLUDES () {
	// Is the reset set or old sql_patches?
	if (((!isWeeklyResetEnabled()) || (!isExtensionInstalledAndNewer('sql_patches', '0.9.8'))) && (isHtmlOutputMode())) {
		// Then abort here
		reportBug(__FUNCTION__, __LINE__, 'Cannot run weekly reset! enabled='.intval(isWeeklyResetEnabled()).',isExtensionInstalledAndNewer()='.intval(isExtensionInstalledAndNewer('sql_patches', '0.9.8')).' Please report this bug. Thanks');
	} // END - if

	// Is ext-sql_patches at least 0.9.8?
	if (isExtensionInstalledAndNewer('sql_patches', '0.9.8')) {
		// Has it changed?
		if ((getLastWeekly() != getWeek()) || (isDebugWeeklyEnabled())) {
			// Include weekly reset scripts
			setIncludePool('weekly', getArrayFromDirectory('inc/weekly/', 'weekly_'));

			// Run the filter
			runFilterChain('load_includes', 'weekly');
		} // END - if
	} // END - if
}

// Load more monthly scripts
function FILTER_RUN_MONTHLY_INCLUDES () {
	// Is the reset set or old sql_patches?
	if (((!isMonthlyResetEnabled()) || (!isExtensionInstalledAndNewer('sql_patches', '0.9.8'))) && (isHtmlOutputMode())) {
		// Then abort here
		reportBug(__FUNCTION__, __LINE__, 'Cannot run monthly reset! enabled='.intval(isMonthlyResetEnabled()).',isExtensionInstalledAndNewer='.intval(isExtensionInstalledAndNewer('sql_patches', '0.9.8')).' Please report this bug. Thanks');
	} // END - if

	// Is ext-sql_patches at least 0.9.8?
	if (isExtensionInstalledAndNewer('sql_patches', '0.9.8')) {
		// Has it changed?
		if ((getLastMonthly() != getMonth()) || (isDebugMonthlyEnabled())) {
			// Include monthly reset scripts
			setIncludePool('monthly', getArrayFromDirectory('inc/monthly/', 'monthly_'));

			// Run the filter
			runFilterChain('load_includes', 'monthly');
		} // END - if
	} // END - if
}

// Load more yearly scripts
function FILTER_RUN_YEARLY_INCLUDES () {
	// Is the reset set or old sql_patches?
	if (((!isYearlyResetEnabled()) || (!isExtensionInstalledAndNewer('sql_patches', '0.9.8'))) && (isHtmlOutputMode())) {
		// Then abort here
		reportBug(__FUNCTION__, __LINE__, 'Cannot run yearly reset! enabled='.intval(isYearlyResetEnabled()).',isExtensionInstalledAndNewer='.intval(isExtensionInstalledAndNewer('sql_patches', '0.9.8')).' Please report this bug. Thanks');
	} // END - if

	// Is ext-sql_patches at least 0.9.8?
	if (isExtensionInstalledAndNewer('sql_patches', '0.9.8')) {
		// Has it changed?
		if ((getLastYearly() != getYear()) || (isDebugYearlyEnabled())) {
			// Include yearly reset scripts
			setIncludePool('yearly', getArrayFromDirectory('inc/yearly/', 'yearly_'));

			// Run the filter
			runFilterChain('load_includes', 'yearly');
		} // END - if
	} // END - if
}

// Filter for removing the given extension
function FILTER_REMOVE_EXTENSION () {
	// Delete this extension (remember to remove it from your server *before* you click on welcome!
	sqlQueryEscaped("DELETE LOW_PRIORITY FROM `{?_MYSQL_PREFIX?}_extensions` WHERE `ext_name`='%s' LIMIT 1",
		array(getCurrentExtensionName()), __FUNCTION__, __LINE__);

	// Remove the extension from cache array as well
	removeExtensionFromArray();

	// Remove the cache
	rebuildCache('extension', 'extension');
}

// Filter for flushing the output
function FILTER_FLUSH_OUTPUT () {
	// Is this filter called?
	if (isset($GLOBALS[__FUNCTION__])) {
		// Abort here
		reportBug(__FUNCTION__, __LINE__, 'Double call of ' . __FUNCTION__ . ' causes problems with sent headers.');
	} // END - if

	// Mark this filter as called
	$GLOBALS[__FUNCTION__] = TRUE;

	// Simple, he?
	outputHtml();
}

// Filter for determining what/action/module
function FILTER_DETERMINE_WHAT_ACTION () {
	// In installation phase we don't have what/action
	if (isInstaller()) {
		// Set both to empty
		setAction('');
		setWhat('');

		// Abort here
		return;
	} // END - if

	// Get all values
	if ((!isCssOutputMode()) && (!isRawOutputMode())) {
		// Fix module
		if (!isModuleSet()) {
			// Is the module registered?
			if (!isModuleRegistered(getRequestElement('module'))) {
				// Is not a valid module
				redirectToUrl('modules.php?module=index');
			} // END - if

			// Is the request element set?
			if (isGetRequestElementSet('module')) {
				// Set module from request
				setModule(getRequestElement('module'));
			} elseif (isHtmlOutputMode()) {
				// Set default module 'index'
				setModule('index');
			} else {
				// Unknown module
				setModule('unknown');
			}
		} // END - if

		// Fix 'what' if not yet set
		if (!isWhatSet()) {
			setWhat(getWhatFromModule(getModule()));
		} // END - if

		// Fix 'action' if not yet set
		if (!isActionSet()) {
			// Is 'what' set?
			if (isWhatSet()) {
				// Set action
				setAction(getActionFromModuleWhat(getModule(), getWhat()));
			} else {
				// Set action/what to empty
				setAction('');
				setWhat('');
			}
		} // END - if
	} else {
		// Set action/what to empty
		setAction('');
		setWhat('');
	}

	// Set default 'what' value
	//* DEBUG: */ debugOutput('-' . getModule() . '/' . getWhat() . '-');
	if ((!isWhatSet()) && (!isActionSet()) && (!isCssOutputMode()) && (!isRawOutputMode())) {
		if (getModule() == 'admin') {
			// Set 'action' value to 'login' in admin menu
			setAction(getActionFromModuleWhat(getModule(), getWhat()));
		} elseif ((getModule() == 'index') || (getModule() == 'login')) {
			// Set 'what' value to 'welcome' in guest and member menu
			setWhatFromConfig('index_home');
		} else {
			// Anything else like begging link
			setWhat('');
		}
	} // END - if
}

// Sends out pooled mails
function FILTER_TRIGGER_SENDING_POOL () {
	// Are we in normal output mode?
	if (!isHtmlOutputMode()) {
		// Only in normal output mode to prevent race-conditons!
		return;
	} // END - if

	// Init counter
	$GLOBALS['pool_cnt'] = '0';

	// Init & set the include pool
	initIncludePool('pool');
	setIncludePool('pool', getArrayFromDirectory('inc/pool/', 'pool-'));

	// Run the filter
	runFilterChain('load_includes', 'pool');

	// Remove the counter
	unset($GLOBALS['pool_cnt']);
}

// Filter for running hourly reset
function FILTER_RUN_HOURLY_RESET () {
	// Only execute this filter if installed
	if ((isInstaller()) || (!isInstalled()) || (!isAdminRegistered()) || (!isExtensionInstalledAndNewer('sql_patches', '0.9.8'))) {
		return;
	} // END - if

	// Shall we run the reset scripts? If a day has changed, maybe also a week/month has changed... Simple! :D
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'getLastHourly()=' . getLastHourly() . ',getHour()=' . getHour());
	if (((getLastHourly() != getHour()) || (isDebugHourlyEnabled())) && (!isInstaller()) && (isAdminRegistered()) && (!isGetRequestElementSet('setup')) && (!isCssOutputMode())) {
		// Tell every module we are in reset-mode!
		doHourly();
	} // END - if
}

// Filter for running daily reset
function FILTER_RUN_DAILY_RESET () {
	// Only execute this filter if installed
	if ((isInstaller()) || (!isInstalled()) || (!isAdminRegistered()) || (!isExtensionInstalledAndNewer('sql_patches', '0.9.8'))) {
		return;
	} // END - if

	// Shall we run the reset scripts? If a day has changed, maybe also a week/month has changed... Simple! :D
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'getLastDaily()=' . getLastDaily() . ',getDay()=' . getDay());
	if (((getLastDaily() != getDay()) || (isDebugDailyEnabled())) && (!isInstaller()) && (isAdminRegistered()) && (!isGetRequestElementSet('setup')) && (!isCssOutputMode())) {
		// Tell every module we are in reset-mode!
		doDaily();
	} // END - if
}

// Filter for running weekly reset
function FILTER_RUN_WEEKLY_RESET () {
	// Only execute this filter if installed
	if ((isInstaller()) || (!isInstalled()) || (!isAdminRegistered()) || (!isExtensionInstalledAndNewer('sql_patches', '0.9.8'))) {
		return;
	} // END - if

	// Shall we run the reset scripts? If a day has changed, maybe also a week/month has changed... Simple! :D
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'getLastWeekly()=' . getLastWeekly() . ',getWeek()=' . getWeek());
	if (((getLastWeekly() != getWeek()) || (isDebugWeeklyEnabled())) && (!isInstaller()) && (isAdminRegistered()) && (!isGetRequestElementSet('setup')) && (!isCssOutputMode())) {
		// Tell every module we are in reset-mode!
		doWeekly();
	} // END - if
}

// Filter for running monthly reset
function FILTER_RUN_MONTHLY_RESET () {
	// Only execute this filter if installed
	if ((isInstaller()) || (!isInstalled()) || (!isAdminRegistered()) || (!isExtensionInstalledAndNewer('sql_patches', '0.9.8'))) {
		return;
	} // END - if

	// Shall we run the reset scripts? If a day has changed, maybe also a week/month has changed... Simple! :D
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'getLastMonthly()=' . getLastMonthly() . ',getMonth()=' . getMonth());
	if (((getLastMonthly() != getMonth()) || (isDebugMonthlyEnabled())) && (!isInstaller()) && (isAdminRegistered()) && (!isGetRequestElementSet('setup')) && (!isCssOutputMode())) {
		// Tell every module we are in reset-mode!
		doMonthly();
	} // END - if
}

// Filter for running yearly reset
function FILTER_RUN_YEARLY_RESET () {
	// Only execute this filter if installed
	if ((isInstaller()) || (!isInstalled()) || (!isAdminRegistered()) || (!isExtensionInstalledAndNewer('sql_patches', '0.9.8'))) {
		return;
	} // END - if

	// Shall we run the reset scripts? If a day has changed, maybe also a week/month has changed... Simple! :D
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'getLastYearly()=' . getLastYearly() . ',getYear()=' . getYear());
	if (((getLastYearly() != getYear()) || (isDebugYearlyEnabled())) && (!isInstaller()) && (isAdminRegistered()) && (!isGetRequestElementSet('setup')) && (!isCssOutputMode())) {
		// Tell every module we are in reset-mode!
		doYearly();
	} // END - if
}

// Filter for loading more runtime includes (not for installation)
function FILTER_LOAD_RUNTIME_INCLUDES () {
	// Load more includes
	foreach (array('databases') as $inc) {
		// Load the include
		loadIncludeOnce('inc/' . $inc . '.php');
	} // END - foreach
}

// Filter for initializing language by loading the language file
function FILTER_INIT_LANGUAGE () {
	// Load language file(s)
	loadLanguageFile();
}

// Filter for checking admin ACL
function FILTER_CHECK_ADMIN_ACL () {
	// Extension not installed so it's always allowed to access everywhere!
	$ret = TRUE;

	// Ok, Cookie-Update done
	if ((isExtensionInstalledAndNewer('admins', '0.3.0')) && (isExtensionActive('admins'))) {
		// Check if action GET variable was set
		$action = getAction();
		if (isWhatSet()) {
			// Get action value by what-value
			$action = getActionFromModuleWhat('admin', getWhat());
		} // END - if

		// Check for access control line of current menu entry
		$ret = isAdminsAllowedByAcl($action, getWhat());
	} // END - if

	// Set it here
	$GLOBALS['acl_allow'] = $ret;
}

// Init random number/cache buster
function FILTER_INIT_RANDOM_NUMBER () {
	// Is the extension ext-sql_patches installed and at least 0.3.6?
	if (isSessionVariableSet('random_number')) {
		// Then set it
		setConfigEntry('RAND_NUMBER', getSession('random_number'));
	} elseif ((isExtensionInstalledAndNewer('sql_patches', '0.3.6')) && (isExtensionInstalledAndNewer('other', '0.2.5'))) {
		// Generate random number
		setConfigEntry('RAND_NUMBER', generateRandomCode(10, mt_rand(10000, 99999), getMemberId(), ''));
	} else {
		// Generate *WEAK* code
		setConfigEntry('RAND_NUMBER', mt_rand(1000000, 9999999));
	}

	// Set it in session for faster script execusion
	setSession('random_number', getConfig('RAND_NUMBER'));

	// Copy it to CACHE_BUSTER
	setConfigEntry('CACHE_BUSTER', getConfig('RAND_NUMBER'));
}

// Update module counter
function FILTER_COUNT_MODULE () {
	// Is installation phase or no admin registered?
	if ((isInstaller()) || (!isInstalled()) || (!isAdminRegistered())) {
		// Then don't count any modules
		return;
	} // END - if

	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'getModule()=' . getModule() . ' - ENTERED!');
	// Count module click
	sqlQueryEscaped("UPDATE `{?_MYSQL_PREFIX?}_mod_reg` SET `clicks`=`clicks`+1 WHERE `module`='%s' LIMIT 1",
		array(getModule()), __FUNCTION__, __LINE__);
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'getModule()=' . getModule() . ',sqlAffectedRows()=' . sqlAffectedRows() . ' - EXIT!');
}

// Handles fatal errors
function FILTER_HANDLE_FATAL_ERRORS () {
	// Is there errors to handle and right output mode?
	if ((!ifFatalErrorsDetected()) || (!isHtmlOutputMode())) {
		// Abort executing here
		return FALSE;
	} // END - if

	// Set content type
	setContentType('text/html');

	// Load config here
	loadIncludeOnce('inc/load_config.php');

	// Default content is none
	$content = '';

	// Installation phase or regular mode?
	if ((isInstaller())) {
		// While we are installing ouput other header than while it is installed... :-)
		$OUT = '';
		foreach (getFatalArray() as $key => $value) {
			// Prepare content for the template
			$content = array(
				'key'   => ($key + 1),
				'value' => $value
			);

			// Load row template
			$OUT .= loadTemplate('install_fatal_row', TRUE, $content);
		}

		// Load main template
		$content = loadTemplate('install_fatal_table', TRUE, $OUT);
	} elseif (isInstalled()) {
		// Display all runtime fatal errors
		$OUT = '';
		foreach (getFatalArray() as $key => $value) {
			// Prepare content for the template
			$content = array(
				'key'   => ($key + 1),
				'value' => $value
			);

			// Load row template
			$OUT .= loadTemplate('runtime_fatal_row', TRUE, $content);
		}

		// Load main template
		$content = loadTemplate('runtime_fatal_table', TRUE, $OUT);
	}

	// Message to regular users (non-admin)
	$CORR = '{--FATAL_REPORT_ERRORS--}';

	// Maybe the admin wants to take a look?
	if (isAdmin()) {
		$CORR = '{--FATAL_CORRECT_ERRORS--}';
	} // END - if

	// Remember all in array
	$content = array(
		'rows' => $content,
		'corr' => $CORR
	);

	// Load header
	loadPageHeader();

	// Load main template
	loadTemplate('fatal_errors', FALSE, $content);

	// Delete all to prevent double-display
	initFatalMessages();

	// Load footer
	loadPageFooter();

	// Abort here
	doShutdown();
}

// Filter for displaying copyright line
function FILTER_DISPLAY_COPYRIGHT () {
	// Shall we display the copyright notice?
	if ((isFullPage()) && ((getWriteFooter() == 'Y') || (isInstalling())) && ($GLOBALS['__header_sent'] == 2) && (ifCopyrightFooterEnabled())) {
		// Backlink enabled?
		if (((isConfigEntrySet('ENABLE_BACKLINK')) && (getConfig('ENABLE_BACKLINK') == 'Y')) || (isInstalling())) {
			// Copyright with backlink, thanks! :-)
			$GLOBALS['__page_footer'] .= loadTemplate('copyright_backlink', TRUE);
		} else {
			// No backlink in Copyright note
			$GLOBALS['__page_footer'] .= loadTemplate('copyright', TRUE);
		}
	} // END - if
}

// Filter for displaying parsing time
function FILTER_DISPLAY_PARSING_TIME () {
	// Shall we display the parsing time and number of queries?
	// 1234                            5                      54    4                    554    4          5543    3                    4432    2                332    2                              21
	if ((((isExtensionInstalledAndNewer('sql_patches', '0.4.1')) && (isShowTimingsEnabled()) && (isFullPage())) && (!isInstaller())) && (isHtmlOutputMode()) && ($GLOBALS['__header_sent'] == 2)) {
		// Then display it here
		displayParsingTime();
	} // END - if
}

// Filter for flushing template cache
function FILTER_FLUSH_TEMPLATE_CACHE () {
	// Do not flush when debugging the template cache
	if (isDebugTemplateCacheEnabled()) {
		// Do not flush templates in debug-mode
		return;
	} // END - if

	// Is there cached eval() data?
	if ((isset($GLOBALS['template_eval'])) && (isFilledArray($GLOBALS['template_eval']))) {
		// Flush all
		foreach ($GLOBALS['template_eval'] as $prefix => $templateArray) {
			foreach ($templateArray as $template => $eval) {
				// Flush the cache (if not yet found)
				flushTemplateCache($prefix, $template, $eval);
			} // END - foreach
		} // END - foreach
	} // END - if
}

// Filter for loading user data
function FILTER_FETCH_USER_DATA ($userid = NULL) {
	// Is the userid not set?
	if (!isValidId($userid)) {
		// Then use member id
		$userid = getMemberId();
	} // END - if

	// Get user data
	if (!fetchUserData($userid)) {
		// Userid is not valid
		reportBug(__FUNCTION__, __LINE__, 'User id ' . $userid . ' is invalid.');
	} // END - if

	// Set member id
	setMemberId($userid);

	// Continue with next filter
	return $userid;
}

// Filter for reseting users' last login failure, only available with latest ext-sql_patches
function FILTER_RESET_USER_LOGIN_FAILURE () {
	// Is the user data valid?
	if (!isMember()) {
		// Do only run for logged in members
		reportBug(__FUNCTION__, __LINE__, 'Please only run this filter for logged in users.');
	} // END - if

	// Remmeber login failures if available
	if (isExtensionInstalledAndNewer('user', '0.3.7')) {
		// Reset login failures
		sqlQueryEscaped("UPDATE
	`{?_MYSQL_PREFIX?}_user_data`
SET
	`login_failures`=0,
	`last_failure`=NULL
WHERE
	`userid`=%s
LIMIT 1",
			array(getMemberId()), __FUNCTION__, __LINE__);

		// Store it in session
		setSession('mailer_member_failures' , getUserData('login_failures'));
		setSession('mailer_member_last_failure', getUserData('last_failure'));
	} // END - if
}

// Try to login the admin by setting some session/cookie variables
function FILTER_DO_LOGIN_ADMIN ($filterData) {
	// Now set all session variables and store the result for later processing
	$GLOBALS['admin_login_success'] = ((
		setAdminMd5(encodeHashForCookie($filterData['pass_hash']))
	) && (
		setAdminId($filterData['id'])
	) && (
		setAdminLast(time())
	));

	// Return the data for further processing
	return $filterData;
}

// Filter for loading page header, this should be ran first!
function FILTER_LOAD_PAGE_HEADER () {
	// Output page header code
	$GLOBALS['__page_header'] = loadTemplate('page_header', TRUE);

	// Include meta data in 'guest' module
	if (isModuleSet()) {
		// Load meta data template
		$GLOBALS['__page_header'] .= loadTemplate(getMenuModeFromModule() . '_metadata', TRUE);

		// Add meta description to header
		if ((isInstalled()) && (isAdminRegistered()) && (isSqlLinkUp()) && (getModule() == 'index')) {
			// Add meta description not in admin and login module and when the script is installed
			generateMetaDescriptionCode();
		} // END - if
	} // END - if
}

// Filter for adding style sheet, closing page header
function FILTER_FINISH_PAGE_HEADER () {
	// Include stylesheet
	loadIncludeOnce('inc/stylesheet.php');

	// Closing HEAD tag
	$GLOBALS['__page_header'] .= '</head>';
}

// Cleans up the DNS cache if ext-sql_patches is at least 0.7.0
function FILTER_CLEANUP_DNS_CACHE () {
	// Is the latest version installed?
	if (isExtensionInstalledAndNewer('sql_patches', '0.7.0')) {
		// Load class file
		loadIncludeOnce('inc/classes/resolver.class.php');

		// Instance the resolver
		$resolver = new HostnameResolver();

		// Purge entries
		$resolver->purgeEntries();

		// Cute, isn't it? ;-)
	} // END - if
}

// Filter for setting CURRENT_DATE, this is required after initialization of extensions
function FILTER_SET_CURRENT_DATE () {
	// Set current date
	setConfigEntry('CURRENT_DATE', generateDateTime(time(), '3'));

	// Epoche time for yesterday, today ... all at 00:00 am
	setConfigEntry('START_YDAY', makeTime(0, 0, 0, time() - getOneDay()));
	setConfigEntry('START_TDAY', makeTime(0, 0, 0, time()));
}

// Filter for marking extension as installed
function FILTER_EXTENSION_MARK_INSTALLED ($filterData) {
	// Mark it as installed
	$GLOBALS['ext_is_installed'][$filterData['ext_name']] = TRUE;
}

// Filter for generating mails links for 'pool' mails
function FILTER_GENERATE_POOL_MAIL_LINKS ($filterData) {
	// Is type 'normal'?
	if ($filterData['type'] == 'normal') {
		// Load template
		$filterData['__output'] .= loadTemplate('admin_links_' . strtolower($filterData['mail_status']) . '_pool_mail', TRUE, $filterData);
	} // END - if

	// Return data
	return $filterData;
}

// Filter to activate exchange
function FILTER_ACTIVATE_EXCHANGE () {
	// Are the extension ext-user/other there?
	if ((!isExtensionActive('user')) || (!isExtensionInstalled('other')) || (isAjaxOutputMode()) || (getActivateXchange() == '0')) {
		// Silently abort here
		return FALSE;
	} // END - if

	// Check total amount of users
	if (getTotalConfirmedUser() >= getActivateXchange()) {
		// Activate System
		addSql("UPDATE `{?_MYSQL_PREFIX?}_mod_reg` SET `locked`='N',`mem_only`='Y' WHERE `module`='order' LIMIT 1");
		addSql("UPDATE `{?_MYSQL_PREFIX?}_member_menu` SET `visible`='Y',`locked`='N' WHERE `what`='order' OR `what`='unconfirmed' LIMIT 2");

		// Run SQLs
		runFilterChain('run_sqls');

		// Update configuration
		updateConfiguration('activate_xchange' ,0);

		// Rebuild cache
		rebuildCache('modules', 'modules');
	} // END - if
}

// Filter to handle configuration
function FILTER_HANDLE_HOME_IN_INDEX_SETTING ($filterData) {
	// Is ext-sql_patches up-to-date?
	if ((isExtensionInstalledAndNewer('sql_patches', '0.8.3')) && (isDisplayHomeInIndexEnabled())) {
		// Is 'search' as same as 'index_home'?
		if ($filterData['search'] == getIndexHome()) {
			// Then set 'content' to link to index.php
			$filterData['content'] = $filterData['prefix'] . '<strong><a class="you_are_here" href="{%url=index.php' . $filterData['link_add'] . '%}">' . getTitleFromMenu($filterData['access_level'], $filterData['search'], $filterData['type'], $filterData['add']) . '</a></strong>';
		} // END - if
	} // END - if

	// Return data
	return $filterData;
}

// Filter to add history entry
function FILTER_ADD_HISTORY_ENTRY ($filterData) {
	// Add the entry
	sqlQueryEscaped("INSERT INTO `{?_MYSQL_PREFIX?}_history` (`history_subject`, `history_userid`, `history_value`) VALUES ('%s',%s,'%s')",
		$filterData,
		__FUNCTION__, __LINE__);

	// Remember insert id for other filters
	$filterData['history_id'] = getSqlInsertId();

	// Return data
	return $filterData;
}

// Filter for initializing ext-sql_patches
function FILTER_GENERATE_FILE_SECRET_HASH ($filterData) {
	// Is ext-sql_patches installed and transfer POINTS word?
	if ((!isExtensionInstalled('sql_patches')) || (!isExtensionInstalledAndNewer('sql_patches', '0.9.0'))) {
		// Extension ext-sql_patches is missing, so better abort here
		return;
	} // END - if

	// Okay, recent enough, so transfer the word for POINTS
	setConfigEntry('POINTS', getPointsWord());

	// Init key
	setConfigEntry('secret_key', '');

	// File hash was generated so we can also file the secret file... hopefully.
	$hashFile = getGenericHashFileName();

	// Read key from secret file
	if ((getFileHash() == '') || (getMasterSalt() == '') || (getPassScramble() == '') || (!isFileReadable($hashFile))) {
		// Maybe need setup of secret key!
		loadIncludeOnce('inc/gen_sql_patches.php');

		// Generate file name again
		$hashFile = getGenericHashFileName();
	} // END - if

	// Test again
	if ((getFileHash() != '') && (getMasterSalt() != '') && (getPassScramble() != '')) {
		// Is the secret key file readable?
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'hashFile=' . $hashFile);
		if (isFileReadable($hashFile)) {
			// Read file
			setConfigEntry('secret_key', readFromFile($hashFile));
		} else {
			// Remove it from database
			updateConfiguration('file_hash', '', '', '0', TRUE);

			// Cannot read secret file!
			reportBug(__FILE__, __LINE__, 'Cannot read secret file! Please try to reload.');
		}
	} // END - if

	// Return data
	return $filterData;
}

// Filter to initialize session
function FILTER_INIT_SESSION ($filterData) {
	// Determine and set referral id
	determineReferralId();

	// Return data
	return $filterData;
}

// Filter for sending "build mail" to admin
function FILTER_SEND_BUILD_MAIL ($filterData) {
	// Was one line updated?
	if (($filterData['affected'] > 0) || (isDebugBuildMailsEnabled())) {
		// Then call the proper function
		call_user_func_array('sendGenericBuildMails', $filterData);
	} // END - if

	// Return data
	return $filterData;
}

// Handles click on referral banner
function FILTER_HANDLE_REFERRER_BANNER_CLICK ($filterData) {
	// Check required parameters
	if ((isGetRequestElementSet('user')) && (isGetRequestElementSet('banner'))) {
		// Update clicks counter...
		sqlQueryEscaped('UPDATE `{?_MYSQL_PREFIX?}_refbanner` SET `clicks`=`clicks`+1 WHERE `id`=%s LIMIT 1',
			array(bigintval(getRequestElement('banner'))), __FUNCTION__, __LINE__);

		// A line has been updated?
		if ((!ifSqlHasZeroAffectedRows()) && (isGetRequestElementSet('user'))) {
			// Redirect to the user's referral link
			redirectToUrl('ref.php?refid=' . bigintval(getRequestElement('user')));
		} // END - if
	} // END - if

	// Return filter data
	return $filterData;
}

// Filter for handling viewing of a referral banner
function FILTER_HANDLE_REFERRER_BANNER_VIEW ($filterData) {
	// Are all required parameters set
	if ((isGetRequestElementSet('user')) && (isGetRequestElementSet('banner'))) {
		// For later things... ;-)
		$result = sqlQueryEscaped('SELECT `url` FROM `{?_MYSQL_PREFIX?}_refbanner` WHERE `id`=%s LIMIT 1',
			array(bigintval(getRequestElement('banner'))), __FUNCTION__, __LINE__);

		// Is there the banner?
		if (sqlNumRows($result) == 1) {
			// Load the URL
			$data = sqlFetchArray($result);

			// Update counter
			sqlQueryEscaped('UPDATE `{?_MYSQL_PREFIX?}_refbanner` SET `counter`=`counter`+1 WHERE `id`=%s LIMIT 1',
				array(bigintval(getRequestElement('banner'))), __FUNCTION__, __LINE__);

			// Set header and ...
			setContentType('image/' . substr($data['url'], -3, 3));

			// Redirect
			redirectToUrl($data['url'], FALSE);
		} // END - if

		// @TODO No banner found, output some default banner
		logDebugMessage(__FUNCTION__, __LINE__, 'Banner ' . bigintval(getRequestElement('banner')) . ' not found.');

		// Free memory
		sqlFreeResult($result);
	} // END - if

	// Do nothing for now
	redirectToUrl('modules.php?module=index');
}

// Filter for redirecting on wrong server name (e.g. bad.your-server.tld will be redirected to your-server.tld)
function FILTER_REDIRECT_WRONG_SERVER_NAME ($filterData) {
	// Get URL
	$url = getUrl();

	// Is the configured URL same as SERVER_NAME?
	if (extractHostnameFromUrl($url) != detectServerName()) {
		// Log all attempts
		logWrongServerNameRedirect();

		// Then redirect to configured URL
		redirectToRequestUri();
	} // END - if

	// Return filter data
	return $filterData;
}

// Filter for adding hidden session id
function FILTER_ADD_INPUT_HIDDEN_SESSION_ID ($filterData) {
	// Is a spider detected?
	if ((!isSpider()) && (!isValidSession()) && ($filterData['__form_method'] == 'get')) {
		// No, then add the hidden field
		$filterData['__replacer'] .= '<input type="hidden" name="' . session_name() . '" value="' . session_id() . '" />';
	} // END - if

	// Return filter data
	return $filterData;
}

// Filter for purging old SERVER_NAME entries
function FILTER_SERVER_NAME_EXTRA_AUTOPURGE ($filterData) {
	// Is purging configured?
	if ((isApServerNameEnabled()) && (getApServerNameSince() > 0)) {
		// Then start purging
		sqlQuery('DELETE LOW_PRIORITY FROM
	`{?_MYSQL_PREFIX?}_server_name_log`
WHERE
	(UNIX_TIMESTAMP() - UNIX_TIMESTAMP(`server_name_added`)) >= {?ap_server_name_since?}',
			__FILE__, __LINE__);

		// Are some entries deleted?
		if (sqlAffectedRows() > 0) {
			// Okay, then send an email to the admin
			sendAdminNotification('{--ADMIN_PURGE_USER_SERVER_NAME_LOG_SUBJECT--}', 'admin_purge_user_server_name_log', sqlAffectedRows());
		} // END - if
	} // END - if

	// Return filter data
	return $filterData;
}

// Filter for detecting menu mode
function FILTER_DETERMINE_MENU_MODE_GENERIC ($filterData) {
	// "Detect" it
	switch (getModule()) {
		case 'chk_login': // Is also faked
		case 'confirm': // Login script is 'member'
		case 'loader': // Is faked to 'member' to avoid indexing by search engines
		case 'mailid': // Mail confirmation is 'member'
			$filterData = 'member';
			break;
	} // END - switch

	// Return filter data
	return $filterData;
}

// Filter to exclude default referral id
function FILTER_EXCLUDE_DEFAULT_REFID ($filterData = array()) {
	// Is it valid?
	if (isValidId(getDefRefid())) {
		// Exclude it
		array_push($filterData, '{?def_refid?}');
	} // END - if

	// Return it
	return $filterData;
}

// [EOF]
?>
