<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 08/31/2003 *
 * ===================                          Last change: 07/02/2004 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : admin.php                                        *
 * -------------------------------------------------------------------- *
 * Short description : Administration module                            *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Administrationsmodul                             *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2015 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	exit();
} // END - if

// Load include file
loadIncludeOnce('inc/modules/admin/admin-inc.php');

// Fix "deleted" cookies in PHP4 (PHP5 does remove them, PHP4 sets them to deleted!)
fixDeletedCookies(array('admin_id', 'admin_md5', 'admin_last'));

// Init return value
$ret = 'init';

// Is no admin registered?
if (!isAdminRegistered()) {
	// Register first admin
	registerFirstAdmin();
} elseif (isGetRequestElementSet('reset_pass')) {
	// Is the form submitted?
	if ((isPostRequestElementSet('send_link')) && (isPostRequestElementSet('email'))) {
		// Output result
		displayMessage(sendAdminPasswordResetLink(postRequestElement('email')));
	} elseif ((isPostRequestElementSet('validate_hash')) && (isPostRequestElementSet('admin_login')) && (isGetRequestElementSet('hash'))) {
		// Validate the login data and hash
		$valid = adminResetValidateHashLogin(getRequestElement('hash'), postRequestElement('admin_login'));

		// Valid?
		if ($valid === TRUE) {
			// Prepare content first
			$content = array(
				'hash'        => getRequestElement('hash'),
				'admin_login' => postRequestElement('admin_login')
			);

			// Validation okay so display form for final password change
			loadTemplate('admin_reset_password_form', FALSE, $content);
		} else {
			// Cannot validate the login data and hash
			displayMessage('{--ADMIN_VALIDATION_RESET_LOGIN_HASH_FAILED--}');
		}
	} elseif ((isPostRequestElementSet('reset_pass')) && (isPostRequestElementSet('hash')) && (isPostRequestElementSet('admin_login')) && (isPostRequestElementSet('admin_password1')) && (postRequestElement('admin_password1') == postRequestElement('admin_password2'))) {
		// Okay, we shall the admin password here. So first revalidate the hash
		if (adminResetValidateHashLogin(postRequestElement('hash'), postRequestElement('admin_login'))) {
			// Output result
			loadTemplate('admin_reset_password_done', FALSE, doResetAdminPassword(postRequestElement('admin_login'), postRequestElement('admin_password1')));
		} else {
			// Validation failed
			displayMessage('{--ADMIN_VALIDATION_RESET_LOGIN_HASH_FAILED2--}');
		}
	} elseif (isGetRequestElementSet('hash')) {
		// Output form for hash validation
		loadTemplate('admin_validate_reset_hash_form', FALSE, getRequestElement('hash'));
	} else {
		// Output reset password form
		loadTemplate('admin_reset_password_send_link');
	}
} elseif ((!isSessionVariableSet('admin_id')) || (!isSessionVariableSet('admin_md5')) || (!isSessionVariableSet('admin_last'))) {
	// At leat one administrator account was created
	if ((isSessionVariableSet('admin_id')) && (isSessionVariableSet('admin_md5')) && (isSessionVariableSet('admin_last'))) {
		// Timeout for last login, we have to logout first!
		redirectToUrl('modules.php?module=admin&amp;logout=1');
	} // END - if

	if (isGetRequestElementSet('setup')) {
		// Registration of first admin is done
		if (getRequestElement('setup') == 'done') {
			// Regisration done!
			displayMessage('{--ADMIN_REGISTER_DONE--}');
		} // END - if
	} // END - if

	// Check if the admin has submitted data or not
	if ((isFormSent()) && ((!isPostRequestElementSet('admin_login')) || (!isPostRequestElementSet('admin_password')) || (strlen(postRequestElement('admin_password')) < getConfig('minium_admin_pass_length')))) {
		setPostRequestElement('login', '***');
	} // END - if

	if ((isFormSent('login')) && (postRequestElement('login') != '***')) {
		// All required data was entered so we check his account
		$ret = ifAdminLoginDataIsValid(postRequestElement('admin_login'), postRequestElement('admin_password'));

		// Which status do we have?
		switch ($ret) {
			case 'done': // Admin and password are okay, so we log in now
				// Load URL
				redirectToUrl('modules.php?' . addAllGetRequestParameters());
				break;

			case '404': // Administrator login not found
				setPostRequestElement('login', $ret);
				$ret = '{%message,ADMIN_ACCOUNT_404=' . postRequestElement('admin_login') . '%}';
				destroyAdminSession(TRUE);
				break;

			case 'password': // Wrong password
				setPostRequestElement('login', $ret);
				$ret = '{--WRONG_PASS--} [<a href="{%url=modules.php?module=admin&amp;reset_pass=1%}">{--ADMIN_RESET_PASS--}</a>]';
				destroyAdminSession(TRUE);
				break;

			default: // Others will be logged
				logDebugMessage(__FILE__, __LINE__, sprintf('Unknown return code %s from ifAdminLoginDataIsValid()', $ret));
				break;
		} // END - switch
	} // END - if

	// Error detected?
	if ($ret != 'done') {
		$content['admin_login'] = '';
		if (isPostRequestElementSet('admin_login')) {
			$content['admin_login'] = postRequestElement('admin_login');
		} // END - if

		// Init array elements
		$content['login_message'] = '';
		$content['pass_message']  = '';

		if (isFormSent('login')) {
			// Set messages to zero
			$loginMessage = '';
			$passwdMessage = '';

			// Check for login
			if (!isPostRequestElementSet('admin_login')) {
				// No login entered?
				$loginMessage = '{--ADMIN_NO_LOGIN--}';
			} elseif ((!empty($ret)) && (postRequestElement('login') == '404')) {
				// An error comes back from login?
				$loginMessage = $ret;
			}

			// Check for password
			if (!isPostRequestElementSet('admin_password')) {
				// No password entered?
				$passwdMessage = '{--ADMIN_NO_PASSWORD--}';
			} elseif ((!empty($ret)) && (postRequestElement('login') == 'password')) {
				// An error comes back from login?
				$passwdMessage = $ret;
			}

			// Load message templates if the messages have been set
			if (!empty($loginMessage)) {
				$content['login_message'] = loadTemplate('admin_login_msg', TRUE, $loginMessage);
			} // END - if
			if (!empty($passwdMessage)) {
				$content['pass_message']  = loadTemplate('admin_login_msg', TRUE, $passwdMessage);
			} // END - if
		} // END - if

		// Add all parameter
		$content['all_parameter'] = addAllGetRequestParameters();

		// Load login form template
		loadTemplate('admin_login_form', FALSE, $content);
	} // END - if
} elseif (isGetRequestElementSet('logout')) {
	// Only try to remove cookies
	if (destroyAdminSession(TRUE)) {
		// Load logout template
		if (isGetRequestElementSet('setup')) {
			// Secure input
			$register = getRequestElement('setup');

			// Special logout redirect for installation of given extension
			loadTemplate(sprintf('admin_logout_%s_install', $register));
		} elseif (isGetRequestElementSet('remove')) {
			// Secure input
			$remove = getRequestElement('remove');

			// Special logout redirect for removal of given extension
			loadTemplate(sprintf('admin_logout_%s_remove', $remove));
		} else {
			// Logged out normally
			loadTemplate('admin_logout');
		}
	} else {
		// Something went wrong here...
		displayErrorMessage('{--ADMIN_LOGOUT_FAILED--}');

		// Add fatal message
		addFatalMessage(__FILE__, __LINE__, '{--CANNOT_UNREG_SESS--}');
	}
} else {
	// Maybe an Admin want's to login?
	$ret = ifAdminCookiesAreValid(getCurrentAdminId(), getAdminMd5());

	// Check status
	switch ($ret) {
		case 'done':
			// Check for access control line of current menu entry
			runFilterChain('check_admin_acl');

			// Check for version and switch between old menu system and new intelligent menu system
			if (adminGetMenuMode() == 'NEW') {
				// Load include for admin AJAX
				loadIncludeOnce('inc/ajax/ajax_admin.php');

				// Load main template
				loadTemplate('admin_ajax_main');
			} else {
				/*
				 * This little call constructs the whole default old and lacky menu system
				 * on left side. It also renders the content on right side
				 */
				doAdminAction();
			}
			break;

		case '404': // Administrator login not found
			setPostRequestElement('login', $ret);
			displayMessage('{%message,ADMIN_ACCOUNT_404=' . getCurrentAdminId() . '%}');
			destroyAdminSession(TRUE);
			break;

		case 'password': // Wrong password
			setPostRequestElement('login', $ret);
			displayMessage('{--WRONG_PASS--}');
			destroyAdminSession(TRUE);
			break;

		case 'session': // Invalid admin session
			setPostRequestElement('login', $ret);
			displayMessage('{--INVALID_ADMIN_SESSION--}');
			destroyAdminSession(TRUE);
			break;

		default: // Others will be logged
			logDebugMessage(__FILE__, __LINE__, sprintf('Unknown return code %s from ifAdminCookiesAreValid()', $ret));
			break;
	} // END - switch
}

// [EOF]
?>
