<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 04/12/2004 *
 * ===================                          Last change: 12/01/2008 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : what-payout.php                                  *
 * -------------------------------------------------------------------- *
 * Short description : Payout requests                                  *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Auszahlungsanfragen                              *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2015 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	// Don't call this directly!
	exit();
} elseif (!isMember()) {
	// Not logged in
	redirectToIndexMemberOnlyModule();
}

// Add description as navigation point
addYouAreHereLink('member', __FILE__);

if ((!isExtensionActive('payout')) && (!isAdmin())) {
	displayMessage('{%pipe,generateExtensionInactiveNotInstalledMessage=payout%}');
	return;
} // END - if

// Translate point into comma
$payoutPoints = getPayoutPoints(getMemberId());

// Output amount
displayMessage('{%message,MEMBER_TOTAL_PAYOUT_POINTS=' . $payoutPoints . '%}');

// GET parameter set?
if (!isGetRequestElementSet('payout')) {
	// Load payout types
	$result = sqlQueryEscaped('SELECT
	`id`,
	`type`,
	`rate`,
	`min_points`,
	`allow_url`
FROM
	`{?_MYSQL_PREFIX?}_payout_types`
WHERE
	%s >= `min_points`
ORDER BY
	`type` ASC',
		array($payoutPoints), __FILE__, __LINE__);

	// Some entries found?
	if (!ifSqlHasZeroNums($result)) {
		// Check for his payouts
		$result_payouts = sqlQueryEscaped('SELECT
	`p`.`id`,
	`p`.`payout_total`,
	`p`.`target_account`,
	`p`.`target_bank`,
	`t`.`type`,
	`p`.`payout_timestamp`,
	`p`.`status`,
	`t`.`allow_url`,
	`p`.`target_url`,
	`p`.`link_text`,
	`p`.`banner_url`
FROM
	`{?_MYSQL_PREFIX?}_user_payouts` AS `p`
LEFT JOIN
	`{?_MYSQL_PREFIX?}_payout_types` AS `t`
ON
	`p`.`payout_id`=`t`.`id`
WHERE
	`p`.`userid`=%s
ORDER BY
	`p`.`payout_timestamp` DESC',
			array(getMemberId()), __FILE__, __LINE__);
		if (!ifSqlHasZeroNums($result_payouts)) {
			// List all his requests
			$OUT = '';
			while ($content = sqlFetchArray($result_payouts)) {
				// Nothing entered must be secured in member/what-payputs.php !
				if ($content['allow_url'] == 'Y') {
					// Banner/Textlink views/clicks request
					if (!empty($content['banner'])) {
						// Banner
						$content['target_account'] = '<img src="' . $content['banner'] . '" alt="' . $content['link_text'] . '" title="' . $content['link_text'] . '" border="0" />';
					} else {
						// Textlink
						$content['target_account'] = $content['link_text'];
					}
					$content['target_bank'] = '<a href="{%pipe,generateDereferrerUrl=' . $content['target_url'] . '%}" target="_blank">{--CLICK_HERE--}</a>';
				} // END - if

				// Prepare data for the template
				$content['payout_timestamp'] = generateDateTime($content['payout_timestamp'], 2);

				// Load row template and switch colors
				$OUT .= loadTemplate('member_payout_row', TRUE, $content);
			} // END - while

			// Load template
			loadTemplate('member_payout', FALSE, $OUT);
		} // END - if

		// Free memory
		sqlFreeResult($result_payouts);

		// Output payout list
		outputPayoutList($payoutPoints);
	} else {
		// No payout types setup
		displayMessage('{--MEMBER_PAYOUT_SETUP_INCOMPLETE_LOW_POINTS--}');
	}

	// Free memory
	sqlFreeResult($result);
} else {
	// Chedk if he can get paid by selected type
	$result = sqlQueryEscaped('SELECT
	`type`,
	`rate`,
	`min_points`,
	`allow_url`
FROM
	`{?_MYSQL_PREFIX?}_payout_types`
WHERE
	`id`=%s
LIMIT 1',
		array(bigintval(getRequestElement('payout'))), __FILE__, __LINE__);

	if (sqlNumRows($result) == 1) {
		// id is valid so load the data
		$content = sqlFetchArray($result);

		// Calculate maximum value
		$max     = round($payoutPoints * $content['rate'] - 0.5);
		$PAY_MAX = '0';

		// Calulcate points from submitted amount
		$points = '0';
		if (isPostRequestElementSet('payout')) {
			$points  = bigintval(postRequestElement('payout')) / $content['rate'];
			$PAY_MAX = $max / $content['rate'];
		} // END - if

		// Has enougth points to payout?
		if ($payoutPoints >= $content['min_points']) {
			// Ok, he can get be paid
			if ((isFormSent()) && ($points <= $PAY_MAX) && ($points >= $content['min_points'])) {
				// Remember points in array
				setPostRequestElement('payout_points', $points);
				setPostRequestElement('type'         , $content['type']);

				// Subtract points from member's account and ignore return status
				subtractPoints('payout', getMemberId(), $points);

				// Add entry to his tranfer history
				if ($content['allow_url'] == 'Y') {
					// Banner/textlink ordered
					sqlQueryEscaped("INSERT INTO `{?_MYSQL_PREFIX?}_user_payouts` (`userid`, `payout_total`, `payout_id`, `payout_timestamp`, `status`, `target_url`, `link_text`, `banner_url`)
VALUES (%s,%s,%s, UNIX_TIMESTAMP(), 'NEW','%s','%s','%s')",
					array(
						getMemberId(),
						bigintval(postRequestElement('payout')),
						bigintval(getRequestElement('payout')),
						postRequestElement('turl'),
						postRequestElement('link_text'),
						postRequestElement('banner')
					), __FILE__, __LINE__);

					// Load templates
					$message_mem = loadEmailTemplate('member_payout_request_banner', postRequestArray(), getMemberId());
					if (isExtensionInstalledAndNewer('admins', '0.4.1')) {
						$adm_tpl = 'admin_payout_request_banner';
					} else {
						$message_adm = loadEmailTemplate('admin_payout_request_banner', postRequestArray(), getMemberId());
					}
				} else {
					// e-currency payout requested
					sqlQueryEscaped("INSERT INTO `{?_MYSQL_PREFIX?}_user_payouts` (`userid`, `payout_total`, `target_account`, `target_bank`, `payout_id`, `payout_timestamp`, `status`, `password`)
VALUES (%s,%s,%s,'%s',%s, UNIX_TIMESTAMP(), 'NEW','%s')",
					array(
						getMemberId(),
						bigintval(postRequestElement('payout')),
						bigintval(postRequestElement('account')),
						postRequestElement('bank'),
						bigintval(getRequestElement('payout')),
						postRequestElement('password')
					), __FILE__, __LINE__);

					// Load templates
					$message_mem = loadEmailTemplate('member_payout_request', postRequestArray(), getMemberId());
					$message_adm = loadEmailTemplate('admin_payout_request', postRequestArray(), getMemberId());
					$admin_tpl = '';

					// @TODO Rewrite this to a filter
					if (isExtensionInstalledAndNewer('admins', '0.4.1')) {
						$admin_tpl = 'admin_payout_request';
					} // END - if
				}

				// Generate task (we ignore the task id here)
				createNewTask('[payout:] {--ADMIN_PAYOUY_REQUEST_SUBJECT--}', $message_adm, 'PAYOUT_REQUEST', getMemberId());

				// Send out mails
				sendEmail(getMemberId(), '{--MEMBER_PAYOUT_REQUEST_SUBJECT--}', $message_mem);

				// To admin(s)
				sendAdminNotification('{--ADMIN_PAYOUY_REQUEST_SUBJECT--}', $admin_tpl, postRequestArray(), getMemberId());

				// Load template and output it
				displayMessage('{--MEMBER_PAYOUT_REQUEST_SENT--}');
			} elseif ($content['allow_url'] == 'Y') {
				// Prepare content
				$content = array(
					'max'    => $max,
					'type'   => $content['type'],
					'payout' => bigintval(getRequestElement('payout'))
				);

				// Generate banner order form
				loadTemplate('member_payout_form_banner', FALSE, $content);
			} else {
				// Prepare content
				$content = array(
					'max'    => $max,
					'type'   => $content['type'],
					'payout' => bigintval(getRequestElement('payout'))
				);

				// Generate normal form
				loadTemplate('member_payout_form', FALSE, $content);
			}
		} else {
			// Not enougth points
			displayMessage('{--MEMBER_PAYOUT_POINTS_NOT_ENOUGTH--}');
		}
	} else {
		// id is invalid
		displayMessage('{--MEMBER_PAYOUT_ID_INVALID--}');
	}

	// Free result
	sqlFreeResult($result);
}

// [EOF]
?>
