<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 09/14/2008 *
 * ===================                          Last change: 09/14/2008 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : purge-general.php                                *
 * -------------------------------------------------------------------- *
 * Short description : General autopurging, nothing extension-specific  *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Allgemeine, nicht erweiterunsabhaengige Auto-    *
 *                     Loeschung                                        *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2015 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} elseif ((!isExtensionActive('autopurge')) || (!isAutoPurgingActive())) {
	// Abort here
	return FALSE;
}

if (getAutoPurge() > 0) {
	// Init SQLs
	initSqls();

	// Init variables
	$admin_points = '0';

	// Then check for outdated mail order. We don't delete them just the confirmation links will be deleted.
	$result = sqlQuery('SELECT
	`id`,
	`userid`,
	`pool_id`,
	`payment_id`
FROM
	`{?_MYSQL_PREFIX?}_user_stats`
WHERE
	(UNIX_TIMESTAMP() - `timestamp_ordered`) >= {?auto_purge?}
ORDER BY
	`userid` ASC', __FILE__, __LINE__);
	if (!ifSqlHasZeroNums($result)) {
		// Start deleting procedure
		$userid = NULL;
		$points = '0';
		while ($content = sqlFetchArray($result)) {
			// Check if confirmation links are purged or not
			$result_links = sqlQueryEscaped("SELECT `id` FROM `{?_MYSQL_PREFIX?}_user_links` WHERE `stats_id`=%s LIMIT 1",
				array(bigintval($content['id'])), __FILE__, __LINE__);

			// Is there one entry?
			if (sqlNumRows($result_links) == 1) {
				// At least one link was found, enougth to pay back the points
				if (($userid != $content['userid']) && (isValidId($userid)) && ($points > 0)) {
					// Directly add points back to senders account
					addPointsAutoPurge($userid, $points);
					$points = '0';
				} // END - if

				// Add points
				$userid = $content['userid'];
				$points += getPaymentPrice($content['payment_id']);
				$admin_points += $content['price'];

				// Remove confirmation links from queue
				addSql(sqlQueryEscaped("DELETE LOW_PRIORITY FROM `{?_MYSQL_PREFIX?}_user_links` WHERE `stats_id`=%s",
					array(bigintval($content['id'])), __FILE__, __LINE__, FALSE));

				// Update status of order
				updatePoolDataById($content['pool_id'], 'data_type', 'DELETED');
			} // END - if

			// Free memory
			sqlFreeResult($result_links);
		} // END - while

		// Are some points collected?
		if (($points > 0) && (isValidId($userid))) {
			// Add last points to last user account
			addPointsAutoPurge($userid, $points);
		} // END - if
	} // END - if

	// Free memory
	sqlFreeResult($result);

	// Is the 'bonus' extension installed and activated?
	if (isExtensionActive('bonus', TRUE)) {
		// Check for bonus campaigns
		$result = sqlQuery("SELECT `id`, `points` FROM `{?_MYSQL_PREFIX?}_bonus` WHERE `data_type`='SEND' AND `timestamp` <= {?auto_purge?} ORDER BY `id` ASC", __FILE__, __LINE__);
		if (!ifSqlHasZeroNums($result)) {
			// Start deleting procedure
			$points = '0';
			while ($content = sqlFetchArray($result)) {
				// Check if confirmation links are purged or not
				$result_links = sqlQueryEscaped("SELECT `id` FROM `{?_MYSQL_PREFIX?}_user_links` WHERE `bonus_id`=%s",
					array(bigintval($content['id'])), __FILE__, __LINE__);
				if (!ifSqlHasZeroNums($result_links)) {
					// At least one link was found, enougth to pay back the points
					$points += $content['points'] * sqlNumRows($result_links);

					// Free memory
					sqlFreeResult($result_links);

					// Remove confirmation links from queue
					addSql(sqlQueryEscaped("DELETE LOW_PRIORITY FROM `{?_MYSQL_PREFIX?}_user_links` WHERE `bonus_id`=%s",
						array(bigintval($content['id'])), __FILE__, __LINE__, FALSE));

					// Update status of order
					addSql(sqlQueryEscaped("UPDATE `{?_MYSQL_PREFIX?}_bonus` SET `data_type`='DELETED' WHERE `id`=%s LIMIT 1",
						array(bigintval($content['id'])), __FILE__, __LINE__, FALSE));
				} // END - if
			} // END - while

			// Is ext-jackpot installed?
			if (isExtensionActive('jackpot')) {
				// Add points to jackpot
				addPointsToJackpot($points);
			} // END - if

			// Add points for the admin
			$admin_points += $points;
		} // END - if

		// Free memory
		sqlFreeResult($result);
	} // END - if

	// Add points from deleted accounts to jackpot, but here just add to notify mail
	if ($admin_points > 0) {
		// Send mail to admin
		sendAdminNotification('{--ADMIN_AUTOPURGE_SUBJECT--}', 'admin_purge_points', $admin_points);
	} // END - if

	// Run all SQLs here
	runFilterChain('run_sqls');
} // END - if

// [EOF]
?>
